@@ -210,8 +210,8 @@ bool ItemSync::updateItem( const Item &storedItem, Item &newItem )
 
   // Check whether the flags differ
   if ( storedItem.flags() != newItem.flags() ) {
-    kDebug( 5250 ) << "Stored flags "  << storedItem.flags()
-                    << "new flags " << newItem.flags();
+    kDebug() << "Stored flags "  << storedItem.flags()
+             << "new flags " << newItem.flags();
     return true;
   }
 
@@ -245,6 +245,8 @@ void ItemSync::Private::slotLocalListDone( KJob * job )
 
   const Item::List list = static_cast<ItemFetchJob*>( job )->items();
   foreach ( const Item &item, list ) {
+    if ( item.remoteId().isEmpty() )
+      continue;
     mLocalItemsById.insert( item.id(), item );
     mLocalItemsByRemoteId.insert( item.remoteId(), item );
     mUnprocessedLocalItems.insert( item );
@@ -296,10 +298,10 @@ void ItemSync::Private::execute()
 void ItemSync::Private::processItems()
 {
   // added / updated
-  foreach ( Item remoteItem, mRemoteItems ) {
+  foreach ( Item remoteItem, mRemoteItems ) { //krazy:exclude=foreach non-const is needed here
 #ifndef NDEBUG
     if ( remoteItem.remoteId().isEmpty() ) {
-      kWarning( 5250 ) << "Item " << remoteItem.id() << " does not have a remote identifier";
+      kWarning() << "Item " << remoteItem.id() << " does not have a remote identifier";
     }
 #endif
 
@@ -339,9 +341,9 @@ void ItemSync::Private::deleteItems( const Item::List &items )
 
     if ( !delItem.isValid() ) {
 #ifndef NDEBUG
-      kWarning( 5250 ) << "Delete item (remoteeId=" << delItem.remoteId()
-                       << "mimeType=" << delItem.mimeType()
-                       << ") does not have a valid UID and no item with that remote ID exists either";
+      kWarning() << "Delete item (remoteeId=" << delItem.remoteId()
+                 << "mimeType=" << delItem.mimeType()
+                 << ") does not have a valid UID and no item with that remote ID exists either";
 #endif
       continue;
     }
