@@ -31,6 +31,7 @@
 
 #include <QtCore/QDebug>
 #include <QtCore/QTimer>
+#include "collectionfetchscope.h"
 
 using namespace Akonadi;
 
@@ -40,6 +41,7 @@ Monitor::Monitor( QObject *parent ) :
     QObject( parent ),
     d_ptr( new MonitorPrivate( this ) )
 {
+  d_ptr->init();
   d_ptr->connectToNotificationManager();
 }
 
@@ -62,6 +64,8 @@ void Monitor::setCollectionMonitored( const Collection &collection, bool monitor
     d->collections << collection;
   else
     d->collections.removeAll( collection );
+
+  emit collectionMonitored( collection, monitored );
 }
 
 void Monitor::setItemMonitored( const Item & item, bool monitored )
@@ -70,6 +74,8 @@ void Monitor::setItemMonitored( const Item & item, bool monitored )
     d->items.insert( item.id() );
   else
     d->items.remove( item.id() );
+
+  emit itemMonitored( item,  monitored );
 }
 
 void Monitor::setResourceMonitored( const QByteArray & resource, bool monitored )
@@ -78,6 +84,8 @@ void Monitor::setResourceMonitored( const QByteArray & resource, bool monitored
     d->resources.insert( resource );
   else
     d->resources.remove( resource );
+
+  emit resourceMonitored( resource, monitored );
 }
 
 void Monitor::setMimeTypeMonitored( const QString & mimetype, bool monitored )
@@ -86,16 +94,21 @@ void Monitor::setMimeTypeMonitored( const QString & mimetype, bool monitored )
     d->mimetypes.insert( mimetype );
   else
     d->mimetypes.remove( mimetype );
+
+  emit mimeTypeMonitored( mimetype, monitored );
 }
 
 void Akonadi::Monitor::setAllMonitored( bool monitored )
 {
   d->monitorAll = monitored;
+
+  emit allMonitored( monitored );
 }
 
 void Monitor::ignoreSession(Session * session)
 {
   d->sessions << session->sessionId();
+  connect( session, SIGNAL(destroyed(QObject*)), this, SLOT(slotSessionDestroyed(QObject*)) );
 }
 
 void Monitor::fetchCollection(bool enable)
@@ -118,6 +131,60 @@ ItemFetchScope &Monitor::itemFetchScope()
   return d->mItemFetchScope;
 }
 
+void Monitor::setCollectionFetchScope( const CollectionFetchScope &fetchScope )
+{
+  d->mCollectionFetchScope = fetchScope;
+}
+
+CollectionFetchScope& Monitor::collectionFetchScope()
+{
+  return d->mCollectionFetchScope;
+}
+
+Collection::List Monitor::collectionsMonitored() const
+{
+  return d->collections;
+}
+
+QList<Item::Id> Monitor::itemsMonitored() const
+{
+  return d->items.toList();
+}
+
+QStringList Monitor::mimeTypesMonitored() const
+{
+  return d->mimetypes.toList();
+}
+
+QList<QByteArray> Monitor::resourcesMonitored() const
+{
+  return d->resources.toList();
+}
+
+bool Monitor::isAllMonitored() const
+{
+  return d->monitorAll;
+}
+
+void Monitor::setSession( Akonadi::Session *session )
+{
+  if (session == d->session)
+    return;
+
+  if (!session)
+    d->session = Session::defaultSession();
+  else
+    d->session = session;
+
+  d->itemCache.setSession(d->session);
+  d->collectionCache.setSession(d->session);
+}
+
+Session* Monitor::session() const
+{
+  return d->session;
+}
+
 #undef d
 
 #include "monitor.moc"
