@@ -17,6 +17,10 @@
     02110-1301, USA.
 */
 
+#include "test_utils.h"
+
+#include <akonadi/agentmanager.h>
+#include <akonadi/agentinstance.h>
 #include <akonadi/control.h>
 #include <akonadi/collection.h>
 #include <akonadi/item.h>
@@ -29,7 +33,7 @@
 #include <QtCore/QObject>
 #include <QSignalSpy>
 
-#include <qtest_kde.h>
+#include <qtest_akonadi.h>
 
 using namespace Akonadi;
 
@@ -39,11 +43,12 @@ class ItemsyncTest : public QObject
 {
   Q_OBJECT
   private:
-    Item::List fetchItems()
+    Item::List fetchItems( const Collection &col )
     {
-      ItemFetchJob *fetch = new ItemFetchJob( Collection( 10 ) );
+      ItemFetchJob *fetch = new ItemFetchJob( col, this );
       fetch->fetchScope().fetchFullPayload();
       fetch->fetchScope().fetchAllAttributes();
+      fetch->fetchScope().setCacheOnly( true ); // resources are switched off anyway
       Q_ASSERT( fetch->exec() );
       Q_ASSERT( !fetch->items().isEmpty() );
       return fetch->items();
@@ -54,25 +59,34 @@ class ItemsyncTest : public QObject
     {
       Control::start();
       qRegisterMetaType<KJob*>();
+
+      // switch all resources offline to reduce interference from them
+      foreach ( Akonadi::AgentInstance agent, Akonadi::AgentManager::self()->instances() )
+        agent.setIsOnline( false );
     }
 
     void testFullSync()
     {
-      Item::List origItems = fetchItems();
+      const Collection col = Collection( collectionIdFromPath( "res1/foo" ) );
+      QVERIFY( col.isValid() );
+      Item::List origItems = fetchItems( col );
 
-      ItemSync* syncer = new ItemSync( Collection( 10 ) );
+      ItemSync* syncer = new ItemSync( col );
       syncer->setFullSyncItems( origItems );
       QVERIFY( syncer->exec() );
 
-      Item::List resultItems = fetchItems();
+      Item::List resultItems = fetchItems( col );
       QCOMPARE( resultItems.count(), origItems.count() );
     }
 
     void testFullStreamingSync()
     {
-      Item::List origItems = fetchItems();
+      const Collection col = Collection( collectionIdFromPath( "res1/foo" ) );
+      QVERIFY( col.isValid() );
+      Item::List origItems = fetchItems( col );
 
-      ItemSync* syncer = new ItemSync( Collection( 10 ) );
+      ItemSync* syncer = new ItemSync( col );
+      syncer->setAutoDelete( false );
       QSignalSpy spy( syncer, SIGNAL(result(KJob*)) );
       QVERIFY( spy.isValid() );
       syncer->setTotalItems( origItems.count() );
@@ -90,22 +104,26 @@ class ItemsyncTest : public QObject
       QTest::qWait( 1000 ); // let it finish its job
       QCOMPARE( spy.count(), 1 );
       KJob *job = spy.at( 0 ).at( 0 ).value<KJob*>();
-      QVERIFY( job );
+      QCOMPARE( job, syncer );
       QCOMPARE( job->error(), 0 );
 
-      Item::List resultItems = fetchItems();
+      Item::List resultItems = fetchItems( col );
       QCOMPARE( resultItems.count(), origItems.count() );
+
+      delete syncer;
     }
 
     void testIncrementalSync()
     {
-      Item::List origItems = fetchItems();
+      const Collection col = Collection( collectionIdFromPath( "res1/foo" ) );
+      QVERIFY( col.isValid() );
+      Item::List origItems = fetchItems( col );
 
-      ItemSync* syncer = new ItemSync( Collection( 10 ) );
+      ItemSync* syncer = new ItemSync( col );
       syncer->setIncrementalSyncItems( origItems, Item::List() );
       QVERIFY( syncer->exec() );
 
-      Item::List resultItems = fetchItems();
+      Item::List resultItems = fetchItems( col );
       QCOMPARE( resultItems.count(), origItems.count() );
 
       Item::List delItems;
@@ -121,18 +139,21 @@ class ItemsyncTest : public QObject
       itemWithRandomRemoteId.setRemoteId( KRandom::randomString( 100 ) );
       delItems << itemWithRandomRemoteId;
 
-      syncer = new ItemSync( Collection( 10 ) );
+      syncer = new ItemSync( col );
       syncer->setIncrementalSyncItems( resultItems, delItems );
 
-      Item::List resultItems2 = fetchItems();
+      Item::List resultItems2 = fetchItems( col );
       QCOMPARE( resultItems2.count(), resultItems.count() );
     }
 
     void testIncrementalStreamingSync()
     {
-      Item::List origItems = fetchItems();
+      const Collection col = Collection( collectionIdFromPath( "res1/foo" ) );
+      QVERIFY( col.isValid() );
+      Item::List origItems = fetchItems( col );
 
-      ItemSync* syncer = new ItemSync( Collection( 10 ) );
+      ItemSync* syncer = new ItemSync( col );
+      syncer->setAutoDelete( false );
       QSignalSpy spy( syncer, SIGNAL(result(KJob*)) );
       QVERIFY( spy.isValid() );
       syncer->setStreamingEnabled( true );
@@ -151,26 +172,30 @@ class ItemsyncTest : public QObject
       QTest::qWait( 1000 ); // let it finish its job
       QCOMPARE( spy.count(), 1 );
       KJob *job = spy.at( 0 ).at( 0 ).value<KJob*>();
-      QVERIFY( job );
+      QCOMPARE( job, syncer );
       QCOMPARE( job->error(), 0 );
 
-      Item::List resultItems = fetchItems();
+      Item::List resultItems = fetchItems( col );
       QCOMPARE( resultItems.count(), origItems.count() );
+
+      delete syncer;
     }
 
     void testEmptyIncrementalSync()
     {
-      Item::List origItems = fetchItems();
+      const Collection col = Collection( collectionIdFromPath( "res1/foo" ) );
+      QVERIFY( col.isValid() );
+      Item::List origItems = fetchItems( col );
 
-      ItemSync* syncer = new ItemSync( Collection( 10 ) );
+      ItemSync* syncer = new ItemSync( col );
       syncer->setIncrementalSyncItems( Item::List(), Item::List() );
       QVERIFY( syncer->exec() );
 
-      Item::List resultItems = fetchItems();
+      Item::List resultItems = fetchItems( col );
       QCOMPARE( resultItems.count(), origItems.count() );
     }
 };
 
-QTEST_KDEMAIN( ItemsyncTest, NoGUI )
+QTEST_AKONADIMAIN( ItemsyncTest, NoGUI )
 
 #include "itemsynctest.moc"
