@@ -59,10 +59,7 @@ using namespace KCal;
 class KCal::CalendarLocal::Private
 {
   public:
-    Private()
-    {
-      mDeletedIncidences.setAutoDelete( true );
-    }
+    Private() {}
     QString mFileName;                     // filename where calendar is stored
     CalFormat *mFormat;                    // calendar format
 
@@ -72,7 +69,6 @@ class KCal::CalendarLocal::Private
     QMultiHash<QString, Todo*>mTodosForDate;// on due dates for all Todos
     QHash<QString, Journal *>mJournals;    // hash on uids of all Journals
     QMultiHash<QString, Journal *>mJournalsForDate; // on dates of all Journals
-    Incidence::List mDeletedIncidences;    // list of all deleted Incidences
 
     void insertEvent( Event *event );
     void insertTodo( Todo *todo );
@@ -168,7 +164,6 @@ void CalendarLocal::close()
   deleteAllTodos();
   deleteAllJournals();
 
-  d->mDeletedIncidences.clearAll();
   setModified( false );
 
   setObserversEnabled( true );
@@ -193,14 +188,13 @@ bool CalendarLocal::deleteEvent( Event *event )
   if ( d->mEvents.remove( uid ) ) {
     setModified( true );
     notifyIncidenceDeleted( event );
-    d->mDeletedIncidences.append( event );
     if ( !event->recurs() ) {
       removeIncidenceFromMultiHashByUID<Event *>(
         d->mEventsForDate, event->dtStart().date().toString(), event->uid() );
     }
     return true;
   } else {
-    kWarning() << "CalendarLocal::deleteEvent(): Event not found.";
+    kWarning() << "Event not found.";
     return false;
   }
 }
@@ -269,14 +263,13 @@ bool CalendarLocal::deleteTodo( Todo *todo )
   if ( d->mTodos.remove( todo->uid() ) ) {
     setModified( true );
     notifyIncidenceDeleted( todo );
-    d->mDeletedIncidences.append( todo );
     if ( todo->hasDueDate() ) {
       removeIncidenceFromMultiHashByUID<Todo *>(
         d->mTodosForDate, todo->dtDue().date().toString(), todo->uid() );
     }
     return true;
   } else {
-    kWarning() << "CalendarLocal::deleteTodo(): Todo not found.";
+    kWarning() << "Todo not found.";
     return false;
   }
 }
@@ -588,12 +581,11 @@ bool CalendarLocal::deleteJournal( Journal *journal )
   if ( d->mJournals.remove( journal->uid() ) ) {
     setModified( true );
     notifyIncidenceDeleted( journal );
-    d->mDeletedIncidences.append( journal );
     removeIncidenceFromMultiHashByUID<Journal *>(
       d->mJournalsForDate, journal->dtStart().date().toString(), journal->uid() );
     return true;
   } else {
-    kWarning() << "CalendarLocal::deleteJournal(): Journal not found.";
+    kWarning() << "Journal not found.";
     return false;
   }
 }
