@@ -94,6 +94,11 @@ class KCAL_EXPORT CalendarResources
         */
         virtual ResourceCalendar *destination( Incidence *incidence ) = 0;
 
+        /**
+	   Return true if we have resources configure. Otherwise returns false.
+           @since 4.3
+        */
+        bool hasCalendarResources();
       protected:
         /**
           Returns the CalendarResourceManager used by this calendar.
@@ -163,7 +168,6 @@ class KCAL_EXPORT CalendarResources
           @param incidence is a pointer to a valid Incidence object.
         */
         ResourceCalendar *destination( Incidence *incidence );
-
       private:
         //@cond PRIVATE
         Q_DISABLE_COPY( AskDestinationPolicy )
@@ -330,6 +334,12 @@ class KCAL_EXPORT CalendarResources
     */
     void setAskDestinationPolicy();
 
+    /**
+      Return true if we have resources configure. Otherwise returns false.
+      @since 4.3
+    */
+    bool hasCalendarResources();
+
     /**
       Returns the current parent for new dialogs. This is a bad hack, but we
       need to properly set the parent for the resource selection dialog.
@@ -475,18 +485,24 @@ class KCAL_EXPORT CalendarResources
       Calendar::rawEvents(const QDate &, const QDate &, const KDateTime::Spec &, bool)
     */
     Event::List rawEvents( const QDate &start, const QDate &end,
-                           const KDateTime::Spec &timespec = KDateTime::Spec(),
+                           const KDateTime::Spec &timeSpec = KDateTime::Spec(),
                            bool inclusive = false );
 
     /**
-      @copydoc
-      Calendar::rawEventsForDate(const QDate &,const KDateTime::Spec &,EventSortField,SortDirection)
+      Returns an unfiltered list of all Events which occur on the given date.
+
+      @param date request unfiltered Event list for this QDate only.
+      @param timeSpec time zone etc. to interpret @p date, or the calendar's
+                      default time spec if none is specified
+      @param sortField specifies the EventSortField.
+      @param sortDirection specifies the SortDirection.
+
+      @return the list of unfiltered Events occurring on the specified QDate.
     */
-    Event::List rawEventsForDate(
-      const QDate &date,
-      const KDateTime::Spec &timespec = KDateTime::Spec(),
-      EventSortField sortField = EventSortUnsorted,
-      SortDirection sortDirection = SortDirectionAscending );
+    Event::List rawEventsForDate( const QDate &date,
+                                  const KDateTime::Spec &timeSpec = KDateTime::Spec(),
+                                  EventSortField sortField = EventSortUnsorted,
+                                  SortDirection sortDirection = SortDirectionAscending );
 
     /**
       @copydoc
@@ -716,6 +732,20 @@ class KCAL_EXPORT CalendarResources
     */
     void slotSaveError( ResourceCalendar *resource, const QString &err );
 
+    /**
+      All addIncidence( Incidence * ), addTodo( Todo * ) addEvent( Event * )
+      and addJournal( Journal * ) calls made between beginAddingIncidences()
+      and endAddingIncidences() will only ask the user to choose a resource once.
+      @since 4.4
+    */
+    void beginAddingIncidences();
+
+    /**
+      @see beginAddingIncidences()
+      @since 4.4
+    */
+    void endAddingIncidences();
+
   private:
     //@cond PRIVATE
     Q_DISABLE_COPY( CalendarResources )
