@@ -709,32 +709,9 @@ class KCal::RecurrenceRule::Private
         mAllDay( false )
     {}
 
-    Private( RecurrenceRule *parent, const Private &p )
-      : mParent( parent ),
-        mRRule( p.mRRule ),
-        mPeriod( p.mPeriod ),
-        mDateStart( p.mDateStart ),
-        mFrequency( p.mFrequency ),
-        mDuration( p.mDuration ),
-        mDateEnd( p.mDateEnd ),
-
-        mBySeconds( p.mBySeconds ),
-        mByMinutes( p.mByMinutes ),
-        mByHours( p.mByHours ),
-        mByDays( p.mByDays ),
-        mByMonthDays( p.mByMonthDays ),
-        mByYearDays( p.mByYearDays ),
-        mByWeekNumbers( p.mByWeekNumbers ),
-        mByMonths( p.mByMonths ),
-        mBySetPos( p.mBySetPos ),
-        mWeekStart( p.mWeekStart ),
-
-        mIsReadOnly( p.mIsReadOnly ),
-        mAllDay( p.mAllDay )
-    {
-        setDirty();
-    }
+    Private( RecurrenceRule *parent, const Private &p );
 
+    Private &operator=( const Private &other );
     bool operator==( const Private &other ) const;
     void clear();
     void setDirty();
@@ -751,6 +728,7 @@ class KCal::RecurrenceRule::Private
                                // unless it matches the rule)
     uint mFrequency;
     /** how often it recurs:
+           < 0 means no end date,
            0 means an explicit end date,
            positive values give the number of occurrences */
     int mDuration;
@@ -783,6 +761,65 @@ class KCal::RecurrenceRule::Private
     uint mTimedRepetition;  // repeats at a regular number of seconds interval, or 0
 };
 
+RecurrenceRule::Private::Private( RecurrenceRule *parent, const Private &p )
+  : mParent( parent ),
+    mRRule( p.mRRule ),
+    mPeriod( p.mPeriod ),
+    mDateStart( p.mDateStart ),
+    mFrequency( p.mFrequency ),
+    mDuration( p.mDuration ),
+    mDateEnd( p.mDateEnd ),
+
+    mBySeconds( p.mBySeconds ),
+    mByMinutes( p.mByMinutes ),
+    mByHours( p.mByHours ),
+    mByDays( p.mByDays ),
+    mByMonthDays( p.mByMonthDays ),
+    mByYearDays( p.mByYearDays ),
+    mByWeekNumbers( p.mByWeekNumbers ),
+    mByMonths( p.mByMonths ),
+    mBySetPos( p.mBySetPos ),
+    mWeekStart( p.mWeekStart ),
+
+    mIsReadOnly( p.mIsReadOnly ),
+    mAllDay( p.mAllDay )
+{
+    setDirty();
+}
+
+RecurrenceRule::Private &RecurrenceRule::Private::operator=( const Private &p )
+{
+  // check for self assignment
+  if ( &p == this ) {
+    return *this;
+  }
+
+  mRRule = p.mRRule;
+  mPeriod = p.mPeriod;
+  mDateStart = p.mDateStart;
+  mFrequency = p.mFrequency;
+  mDuration = p.mDuration;
+  mDateEnd = p.mDateEnd;
+
+  mBySeconds = p.mBySeconds;
+  mByMinutes = p.mByMinutes;
+  mByHours = p.mByHours;
+  mByDays = p.mByDays;
+  mByMonthDays = p.mByMonthDays;
+  mByYearDays = p.mByYearDays;
+  mByWeekNumbers = p.mByWeekNumbers;
+  mByMonths = p.mByMonths;
+  mBySetPos = p.mBySetPos;
+  mWeekStart = p.mWeekStart;
+
+  mIsReadOnly = p.mIsReadOnly;
+  mAllDay = p.mAllDay;
+
+  setDirty();
+
+  return *this;
+}
+
 bool RecurrenceRule::Private::operator==( const Private &r ) const
 {
   return
@@ -862,6 +899,18 @@ bool RecurrenceRule::operator==( const RecurrenceRule &r ) const
   return *d == *r.d;
 }
 
+RecurrenceRule &RecurrenceRule::operator=( const RecurrenceRule &r )
+{
+  // check for self assignment
+  if ( &r == this ) {
+    return *this;
+  }
+
+  *d = *r.d;
+
+  return *this;
+}
+
 void RecurrenceRule::addObserver( RuleObserver *observer )
 {
   if ( !d->mObservers.contains( observer ) ) {
@@ -1693,6 +1742,8 @@ DateTimeList RecurrenceRule::timesInInterval( const KDateTime &dtStart,
     KDateTime dt = start.addSecs( d->mTimedRepetition - n );
     if ( dt < enddt ) {
       n = static_cast<int>( ( dt.secsTo_long( enddt ) - 1 ) / d->mTimedRepetition ) + 1;
+      // limit n by a sane value else we can "explode".
+      n = qMin( n, LOOP_LIMIT );
       for ( int i = 0;  i < n;  dt = dt.addSecs( d->mTimedRepetition ), ++i ) {
         result += dt;
       }
@@ -2050,6 +2101,7 @@ QString dumpTime( const KDateTime &dt )
   }
   return result;
 #else
+  Q_UNUSED( dt );
   return QString();
 #endif
 }
