@@ -56,7 +56,6 @@ imap://server/folder/
  *  ERR_DOES_NOT_EXIST is reserved for folders.
  */
 
-#include <kdepimlibs-compat.h> // for KDE_signal, remove in KDEPIM 4.2
 #include "imap4.h"
 
 #include <QByteArray>
@@ -70,11 +69,9 @@ imap://server/folder/
 #include <sys/wait.h>
 #include <errno.h>
 
-#ifdef HAVE_LIBSASL2
 extern "C" {
 #include <sasl/sasl.h>
 }
-#endif
 
 #include <qbuffer.h>
 #include <qdatetime.h>
@@ -119,10 +116,8 @@ kdemain (int argc, char **argv)
     ::exit (-1);
   }
 
-#ifdef HAVE_LIBSASL2
   if (!initSASL())
     ::exit(-1);
-#endif
 
   //set debug handler
 
@@ -136,9 +131,7 @@ kdemain (int argc, char **argv)
   slave->dispatchLoop ();
   delete slave;
 
-#ifdef HAVE_LIBSASL2
   sasl_done();
-#endif
 
   return 0;
 }
@@ -889,7 +882,7 @@ IMAP4Protocol::put (const KUrl & _url, int, KIO::JobFlags)
           QString uid = cmd->resultInfo();
           if ( uid.contains("APPENDUID") )
           {
-            uid = uid.section(" ", 2, 2);
+            uid = uid.section(' ', 2, 2);
             uid.truncate(uid.length()-1);
             infoMessage("UID "+uid);
           }
@@ -1063,7 +1056,7 @@ IMAP4Protocol::copy (const KUrl & src, const KUrl & dest, int, KIO::JobFlags fla
         QString uid = cmd->resultInfo();
         if ( uid.contains("COPYUID") )
         {
-          uid = uid.section(" ", 2, 3);
+          uid = uid.section(' ', 2, 3);
           uid.truncate(uid.length()-1);
           infoMessage("UID "+uid);
         }
@@ -1321,7 +1314,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     if ( hasCapability( "ACL" ) ) {
       specialACLCommand( cmd, stream );
     } else {
-      error( ERR_UNSUPPORTED_ACTION, "ACL" );
+      error( ERR_UNSUPPORTED_ACTION, QString::fromLatin1("ACL") );
     }
     break;
   }
@@ -1333,7 +1326,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     if ( hasCapability( "ANNOTATEMORE" ) ) {
       specialAnnotateMoreCommand( cmd, stream );
     } else {
-      error( ERR_UNSUPPORTED_ACTION, "ANNOTATEMORE" );
+      error( ERR_UNSUPPORTED_ACTION, QString::fromLatin1("ANNOTATEMORE") );
     }
     break;
   }
@@ -1345,7 +1338,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     if ( hasCapability( "QUOTA" ) ) {
       specialQuotaCommand( cmd, stream );
     } else {
-      error( ERR_UNSUPPORTED_ACTION, "QUOTA" );
+      error( ERR_UNSUPPORTED_ACTION, QString::fromLatin1("QUOTA") );
     }
     break;
   }
@@ -1415,7 +1408,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     if (cmd->result () != "OK")
     {
       completeQueue.removeAll (cmd);
-      error(ERR_COULD_NOT_WRITE,
+      error(ERR_SLAVE_DEFINED,
             i18n( "Changing the flags of message %1 failed.", _url.prettyUrl() ) );
       return;
     }
@@ -1553,7 +1546,7 @@ IMAP4Protocol::specialSearchCommand( QDataStream& stream )
   stream >> _url;
   QString aBox, aSequence, aLType, aSection, aValidity, aDelimiter, aInfo;
   parseURL (_url, aBox, aSection, aLType, aSequence, aValidity, aDelimiter, aInfo);
-  if (!assureBox(aBox, false)) return;
+  if (!assureBox(aBox, true)) return;
 
   CommandPtr cmd = doCommand (imapCommand::clientSearch( aSection ));
   if (cmd->result () != "OK")
@@ -1790,7 +1783,7 @@ IMAP4Protocol::rename (const KUrl & src, const KUrl & dest, KIO::JobFlags flags)
           completeQueue.removeAll(cmd);
           if (!ok)
           {
-            error(ERR_CANNOT_RENAME, i18n("Unable to close mailbox."));
+            error(ERR_SLAVE_DEFINED, i18n("Unable to close mailbox."));
             return;
           }
           setState(ISTATE_LOGIN);
@@ -1858,7 +1851,7 @@ IMAP4Protocol::stat (const KUrl & _url)
       completeQueue.removeAll(cmd);
       if (!ok)
       {
-        error(ERR_COULD_NOT_STAT, i18n("Unable to close mailbox."));
+        error(ERR_SLAVE_DEFINED, i18n("Unable to close mailbox."));
         return;
       }
       setState(ISTATE_LOGIN);
@@ -1888,7 +1881,7 @@ IMAP4Protocol::stat (const KUrl & _url)
       }
       completeQueue.removeAll (cmd);
       if (found)
-        error(ERR_COULD_NOT_STAT, i18n("Unable to get information about folder %1. The server replied: %2", aBox, cmdInfo));
+        error(ERR_SLAVE_DEFINED, i18n("Unable to get information about folder %1. The server replied: %2", aBox, cmdInfo));
       else
         error(KIO::ERR_DOES_NOT_EXIST, aBox);
       return;
@@ -2100,6 +2093,18 @@ bool IMAP4Protocol::makeLogin ()
       removeCapability( "ANNOTATEMORE" );
     }
 
+    // starting from Cyrus IMAP 2.3.9, shared seen flags are available
+    QRegExp regExp( "Cyrus\\sIMAP[4]{0,1}\\sv(\\d+)\\.(\\d+)\\.(\\d+)", Qt::CaseInsensitive );
+    if ( regExp.indexIn( greeting ) >= 0 ) {
+      const int major = regExp.cap( 1 ).toInt();
+      const int minor = regExp.cap( 2 ).toInt();
+      const int patch = regExp.cap( 3 ).toInt();
+      if ( major > 2 || (major == 2 && (minor > 3 || (minor == 3 && patch > 9))) ) {
+        kDebug(7116) << "Cyrus IMAP >= 2.3.9 detected, enabling shared seen flag support";
+        imapCapabilities.append( "x-kmail-sharedseen" );
+      }
+    }
+
     kDebug(7116) <<"IMAP4::makeLogin - attempting login";
 
     KIO::AuthInfo authInfo;
@@ -2119,21 +2124,17 @@ bool IMAP4Protocol::makeLogin ()
         }
       }
       if (!clientLogin (myUser, myPass, resultInfo))
-        error(KIO::ERR_COULD_NOT_AUTHENTICATE, i18n("Unable to login. Probably the "
+        error(ERR_SLAVE_DEFINED, i18n("Unable to login. Probably the "
         "password is wrong.\nThe server %1 replied:\n%2", myHost, resultInfo));
     }
     else
     {
-#ifdef HAVE_LIBSASL2
       if (!clientAuthenticate (this, authInfo, myHost, myAuth, mySSL, resultInfo))
-        error(KIO::ERR_COULD_NOT_AUTHENTICATE, i18n("Unable to authenticate via %1.\n"	"The server %2 replied:\n%3", myAuth, myHost, resultInfo));
+        error(ERR_SLAVE_DEFINED, i18n("Unable to authenticate via %1.\n"	"The server %2 replied:\n%3", myAuth, myHost, resultInfo));
       else {
         myUser = authInfo.username;
         myPass = authInfo.password;
       }
-#else
-      error(KIO::ERR_COULD_NOT_LOGIN, i18n("SASL authentication is not compiled into kio_imap4."));
-#endif
     }
     if ( hasCapability("NAMESPACE") )
     {
@@ -2499,7 +2500,7 @@ IMAP4Protocol::parseURL (const KUrl & _url, QString & _box,
         << "from URL" << _url.path();
     }
     if (_hierarchyDelimiter.isEmpty())
-      _hierarchyDelimiter = "/";
+      _hierarchyDelimiter = '/';
   }
   kDebug(7116) <<"IMAP4::parseURL - return" << retVal;
 
@@ -2547,9 +2548,9 @@ void IMAP4Protocol::flushOutput(const QString &contentEncoding)
   {
     // get the coding from the MIME header
     QByteArray decoded;
-    if ( contentEncoding.startsWith("quoted-printable", Qt::CaseInsensitive) )
+    if ( contentEncoding.startsWith(QLatin1String("quoted-printable"), Qt::CaseInsensitive) )
       decoded = KCodecs::quotedPrintableDecode(outputCache);
-    else if ( contentEncoding.startsWith("base64", Qt::CaseInsensitive) )
+    else if ( contentEncoding.startsWith(QLatin1String("base64"), Qt::CaseInsensitive) )
       decoded = QByteArray::fromBase64( outputCache );
     else
       decoded = outputCache;
