@@ -1,7 +1,7 @@
 /*  -*- c++ -*-
     kmime_header_parsing.cpp
 
-    KMime, the KDE internet mail/usenet news message library.
+    KMime, the KDE Internet mail/usenet news message library.
     Copyright (c) 2001-2002 Marc Mutz <mutz@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -23,6 +23,8 @@
 #include "kmime_header_parsing.h"
 
 #include "kmime_codecs.h"
+#include "kmime_headerfactory_p.h"
+#include "kmime_headers.h"
 #include "kmime_util.h"
 #include "kmime_dateformatter.h"
 #include "kmime_warning.h"
@@ -45,6 +47,16 @@ namespace KMime {
 
 namespace Types {
 
+// QUrl::fromAce is extremely expensive, so only use it when necessary.
+// Fortunately, the presence of IDNA is readily detected with a substring match...
+static inline QString QUrl_fromAce_wrapper( const QString & domain )
+{
+    if ( domain.contains( QLatin1String( "xn--" ) ) )
+        return QUrl::fromAce( domain.toLatin1() );
+    else
+        return domain;
+}
+
 static QString addr_spec_as_string( const AddrSpec & as, bool pretty )
 {
   if ( as.isEmpty() ) {
@@ -66,9 +78,12 @@ static QString addr_spec_as_string( const AddrSpec & as, bool pretty )
       result += ch;
     }
   }
-  const QString dom = pretty ? QUrl::fromAce( as.domain.toLatin1() ) : as.domain ;
+  const QString dom = pretty ? QUrl_fromAce_wrapper( as.domain ) : as.domain ;
   if ( needsQuotes ) {
-    return '"' + result + "\"@" + dom;
+    result = '"' + result + "\"";
+  }
+  if( dom.isEmpty() ) {
+    return result;
   } else {
     return result + '@' + dom;
   }
@@ -124,14 +139,14 @@ void Mailbox::setAddress( const QByteArray &addr )
 
 void Mailbox::setName( const QString &name )
 {
-  mDisplayName = name;
+  mDisplayName = removeBidiControlChars( name );
 }
 
 void Mailbox::setNameFrom7Bit( const QByteArray &name,
                                const QByteArray &defaultCharset )
 {
   QByteArray cs;
-  mDisplayName = decodeRFC2047String( name, cs, defaultCharset, false );
+  setName( decodeRFC2047String( name, cs, defaultCharset, false ) );
 }
 
 bool Mailbox::hasAddress() const
@@ -207,8 +222,8 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
   char ch = *scursor++;
 
   if ( ch != '?' ) {
-    // kDebug(5320) << "first";
-    KMIME_WARN_PREMATURE_END_OF( EncodedWord );
+    // kDebug() << "first";
+    //KMIME_WARN_PREMATURE_END_OF( EncodedWord );
     return false;
   }
 
@@ -229,7 +244,7 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
 
   // not found? can't be an encoded-word!
   if ( scursor == send || *scursor != '?' ) {
-    // kDebug(5320) << "second";
+    // kDebug() << "second";
     KMIME_WARN_PREMATURE_END_OF( EncodedWord );
     return false;
   }
@@ -260,7 +275,7 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
 
   // not found? Can't be an encoded-word!
   if ( scursor == send || *scursor != '?' ) {
-    // kDebug(5320) << "third";
+    // kDebug() << "third";
     KMIME_WARN_PREMATURE_END_OF( EncodedWord );
     return false;
   }
@@ -268,7 +283,7 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
   // extract the encoding information:
   QByteArray maybeEncoding( encodingStart, scursor - encodingStart );
 
-  // kDebug(5320) << "parseEncodedWord: found charset == \"" << maybeCharset
+  // kDebug() << "parseEncodedWord: found charset == \"" << maybeCharset
   //         << "\"; language == \"" << maybeLanguage
   //         << "\"; encoding == \"" << maybeEncoding << "\"";
 
@@ -281,28 +296,31 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
   scursor++;
   const char * encodedTextStart = scursor;
 
-  // find next '?' (ending the encoded-text):
+  // find the '?=' sequence (ending the encoded-text):
   for ( ; scursor != send ; scursor++ ) {
     if ( *scursor == '?' ) {
-      break;
+      if ( scursor + 1 != send ) {
+        if ( *( scursor + 1 ) != '=' ) { // We expect a '=' after the '?', but we got something else; ignore
+          KMIME_WARN << "Stray '?' in q-encoded word, ignoring this.";
+          continue;
+        }
+        else { // yep, found a '?=' sequence
+          scursor += 2;
+          break;
+        }
+      }
+      else { // The '?' is the last char, but we need a '=' after it!
+        KMIME_WARN_PREMATURE_END_OF( EncodedWord );
+        return false;
+      }
     }
   }
 
-  // not found? Can't be an encoded-word!
-  // ### maybe evaluate it nonetheless if the rest is OK?
-  if ( scursor == send || *scursor != '?' ) {
-    // kDebug(5320) << "fourth";
+  if ( *( scursor - 2 ) != '?' || *( scursor - 1 ) != '=' ||
+       scursor < encodedTextStart + 2 ) {
     KMIME_WARN_PREMATURE_END_OF( EncodedWord );
     return false;
   }
-  scursor++;
-  // check for trailing '=':
-  if ( scursor == send || *scursor != '=' ) {
-    // kDebug(5320) << "fifth";
-    KMIME_WARN_PREMATURE_END_OF( EncodedWord );
-    return false;
-  }
-  scursor++;
 
   // set end sentinel for encoded-text:
   const char * const encodedTextEnd = scursor - 2;
@@ -345,28 +363,28 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
     return false;
   };
 
-  // kDebug(5320) << "mimeName(): \"" << textCodec->name() << "\"";
+  // kDebug() << "mimeName(): \"" << textCodec->name() << "\"";
 
   // allocate a temporary buffer to store the 8bit text:
   int encodedTextLength = encodedTextEnd - encodedTextStart;
   QByteArray buffer;
   buffer.resize( codec->maxDecodedSizeFor( encodedTextLength ) );
-  QByteArray::Iterator bit = buffer.begin();
-  QByteArray::ConstIterator bend = buffer.end();
+  char *bbegin = buffer.data();
+  char *bend = bbegin + buffer.length();
 
   //
   // STEP 5:
   // do the actual decoding
   //
 
-  if ( !dec->decode( encodedTextStart, encodedTextEnd, bit, bend ) ) {
+  if ( !dec->decode( encodedTextStart, encodedTextEnd, bbegin, bend ) ) {
     KMIME_WARN << codec->name() << "codec lies about its maxDecodedSizeFor("
                << encodedTextLength << ")\nresult may be truncated";
   }
 
-  result = textCodec->toUnicode( buffer.begin(), bit - buffer.begin() );
+  result = textCodec->toUnicode( buffer.data(), bbegin - buffer.data() );
 
-  // kDebug(5320) << "result now: \"" << result << "\"";
+  // kDebug() << "result now: \"" << result << "\"";
   // cleanup:
   delete dec;
   language = maybeLanguage;
@@ -906,6 +924,11 @@ bool parseDomain( const char* &scursor, const char * const send,
     QString maybeDotAtom;
     if ( parseDotAtom( scursor, send, maybeDotAtom, isCRLF ) ) {
       result = maybeDotAtom;
+      // Domain may end with '.', if so preserve it'
+      if ( scursor != send && *scursor == '.' ) {
+        result += QChar('.');
+        scursor++;
+      }
       return true;
     }
   }
@@ -1174,7 +1197,9 @@ bool parseGroup( const char* &scursor, const char * const send,
     return false;
   }
 
-  result.displayName = maybeDisplayName;
+  // KDE5 TODO: Don't expose displayName as public, but rather add setter for it that
+  //            automatically calls removeBidiControlChars
+  result.displayName = removeBidiControlChars( maybeDisplayName );
 
   // get obs-mbox-list (may contain empty entries):
   scursor++;
@@ -1538,7 +1563,7 @@ static void decodeRFC2231Value( Codec* &rfc2231Codec,
 
   value += textcodec->toUnicode( buffer.begin(), bit - buffer.begin() );
 
-  // kDebug(5320) << "value now: \"" << value << "\"";
+  // kDebug() << "value now: \"" << value << "\"";
   // cleanup:
   delete dec;
 }
@@ -1655,7 +1680,7 @@ bool parseParameterList( const char* &scursor, const char * const send,
   return true;
 }
 
-static const char * stdDayNames[] = {
+static const char * const stdDayNames[] = {
   "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
 };
 static const int stdDayNamesLen = sizeof stdDayNames / sizeof *stdDayNames;
@@ -1670,7 +1695,7 @@ static bool parseDayName( const char* &scursor, const char * const send )
   for ( int i = 0 ; i < stdDayNamesLen ; ++i ) {
     if ( qstrnicmp( scursor, stdDayNames[i], 3 ) == 0 ) {
       scursor += 3;
-      // kDebug(5320) << "found" << stdDayNames[i];
+      // kDebug() << "found" << stdDayNames[i];
       return true;
     }
   }
@@ -1678,7 +1703,7 @@ static bool parseDayName( const char* &scursor, const char * const send )
   return false;
 }
 
-static const char * stdMonthNames[] = {
+static const char * const stdMonthNames[] = {
   "Jan", "Feb", "Mar", "Apr", "May", "Jun",
   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
 };
@@ -2020,6 +2045,86 @@ bool parseDateTime( const char* &scursor, const char * const send,
   return true;
 }
 
+Headers::Base *extractFirstHeader( QByteArray &head )
+{
+  int endOfFieldBody = 0;
+  int len = head.length() - 1;
+  bool folded = false;
+  Headers::Base *header = 0;
+
+  int startOfFieldBody = head.indexOf( ":" );
+  const int endOfFieldHeader = startOfFieldBody;
+
+  if ( startOfFieldBody > -1 ) {    //there is another header
+    startOfFieldBody++; //skip the ':'
+    if ( head[startOfFieldBody] == ' ' ) { // skip the space after the ':', if there
+      startOfFieldBody++;
+    }
+    endOfFieldBody = startOfFieldBody;
+
+    // If the first line contains nothing, but the next line starts with a space
+    // or a tab, that means a stupid mail client has made the first header field line
+    // entirely empty, and has folded the rest to the next line(s).
+    if ( head[endOfFieldBody] == '\n' && endOfFieldBody + 1 < len &&
+         ( head[endOfFieldBody + 1] == ' ' ||
+           head[endOfFieldBody + 1] == '\t' ) ) {
+
+      // Skip \n and first whitespace
+      startOfFieldBody += 2;
+      endOfFieldBody += 2;
+    }
+    
+    if ( head[endOfFieldBody] != '\n' ) {  // check if the header is not empty
+      while ( 1 ) {
+        endOfFieldBody = head.indexOf( '\n', endOfFieldBody + 1 );
+        if ( endOfFieldBody == -1 || endOfFieldBody == len ||
+              ( head[endOfFieldBody+1] != ' ' &&
+                head[endOfFieldBody+1] != '\t' ) ) {
+          //break if we reach the end of the string, honor folded lines
+          break;
+        } else {
+          folded = true;
+        }
+      }
+    }
+
+    if ( endOfFieldBody < 0 ) {
+      endOfFieldBody = len + 1; //take the rest of the string
+    }
+
+    QByteArray rawType = head.left( endOfFieldHeader );
+    QByteArray rawFieldBody = head.mid( startOfFieldBody, endOfFieldBody - startOfFieldBody );
+    if ( folded ) {
+      rawFieldBody = unfoldHeader( rawFieldBody );
+    }
+    header = HeaderFactory::self()->createHeader( rawType );
+    if( !header ) {
+      //kWarning() << "Returning Generic header of type" << rawType;
+      header = new Headers::Generic( rawType );
+    }
+    header->from7BitString( rawFieldBody );
+
+    head.remove( 0, endOfFieldBody + 1 );
+  } else {
+    head.clear();
+  }
+
+  return header;
+}
+
+Headers::Base::List parseHeaders( const QByteArray &head )
+{
+  Headers::Base::List ret;
+  Headers::Base *h;
+
+  QByteArray copy = head;
+  while( ( h = extractFirstHeader( copy ) ) ) {
+    ret << h;
+  }
+
+  return ret;
+}
+
 } // namespace HeaderParsing
 
 } // namespace KMime
